@@ -1,3 +1,23 @@
+/*
+ * Copyright 2018-present MongoDB, Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
+#if !defined(BSON_INSIDE) && !defined(BSON_COMPILATION)
+#error "Only <bson/bson.h> can be included directly."
+#endif
+
 #ifndef BSON_CONFIG_H
 #define BSON_CONFIG_H
 
@@ -8,6 +28,7 @@
  */
 #include <Python.h>
 
+
 /*
  * Define to 1234 for Little Endian, 4321 for Big Endian.
  */
@@ -17,6 +38,7 @@
 # define BSON_BYTE_ORDER 1234
 #endif
 
+
 /*
  * Define to 1 if you have stdbool.h
  */
@@ -36,36 +58,6 @@
 #endif
 
 
-/*
- * Define to 1 if we have access to GCC 32-bit atomic builtins.
- * While this requires GCC 4.1+ in most cases, it is also architecture
- * dependent. For example, some PPC or ARM systems may not have it even
- * if it is a recent GCC version.
- */
-#define BSON_HAVE_ATOMIC_32_ADD_AND_FETCH 0
-#if BSON_HAVE_ATOMIC_32_ADD_AND_FETCH != 1
-# undef BSON_HAVE_ATOMIC_32_ADD_AND_FETCH
-#endif
-
-/*
- * Similarly, define to 1 if we have access to GCC 64-bit atomic builtins.
- */
-#define BSON_HAVE_ATOMIC_64_ADD_AND_FETCH 0
-#if BSON_HAVE_ATOMIC_64_ADD_AND_FETCH != 1
-# undef BSON_HAVE_ATOMIC_64_ADD_AND_FETCH
-#endif
-
-
-/*
- * Define to 1 if your system requires {} around PTHREAD_ONCE_INIT.
- * This is typically just Solaris 8-10.
- */
-#define BSON_PTHREAD_ONCE_INIT_NEEDS_BRACES 0
-#if BSON_PTHREAD_ONCE_INIT_NEEDS_BRACES != 1
-# undef BSON_PTHREAD_ONCE_INIT_NEEDS_BRACES
-#endif
-
-
 /*
  * Define to 1 if you have clock_gettime() available.
  */
@@ -78,6 +70,15 @@
 #endif
 
 
+/*
+ * Define to 1 if you have strings.h available on your platform.
+ */
+#define BSON_HAVE_STRINGS_H 0
+#if BSON_HAVE_STRINGS_H != 1
+# undef BSON_HAVE_STRINGS_H
+#endif
+
+
 /*
  * Define to 1 if you have strnlen available on your platform.
  */
@@ -90,7 +91,9 @@
 /*
  * Define to 1 if you have snprintf available on your platform.
  */
-#ifndef MS_WINDOWS
+#ifdef MS_WINDOWS
+# define BSON_HAVE_SNPRINTF 0
+#else
 # define BSON_HAVE_SNPRINTF 1
 #endif
 
@@ -100,24 +103,18 @@
 
 
 /*
- * Define to 1 if you have reallocf available on your platform.
+ * Define to 1 if you have gmtime_r available on your platform.
  */
-#define BSON_HAVE_REALLOCF 0
-#if BSON_HAVE_REALLOCF != 1
-# undef BSON_HAVE_REALLOCF
+#ifdef MS_WINDOWS
+# define BSON_HAVE_GMTIME_R 0
+#else
+# define BSON_HAVE_GMTIME_R 1
 #endif
 
-
-/*
- * Define to 1 if you have _set_output_format (VS2013 and older).
- */
-#if defined(_MSV_VER) && (_MSC_VER < 1900)
-# define BSON_NEEDS_SET_OUTPUT_FORMAT 1
+#if BSON_HAVE_GMTIME_R != 1
+# undef BSON_HAVE_GMTIME_R
 #endif
 
-#if BSON_NEEDS_SET_OUTPUT_FORMAT != 1
-# undef BSON_NEEDS_SET_OUTPUT_FORMAT
-#endif
 
 /*
  * Define to 1 if you have struct timespec available on your platform.
@@ -149,12 +146,19 @@
 #endif
 
 
-/*
- * Define to 1 if _Decimal128 (BID format) is available on your platform.
- */
-#define BSON_HAVE_DECIMAL128 0
-#if BSON_HAVE_DECIMAL128 != 1
-# undef BSON_HAVE_DECIMAL128
+#ifdef MS_WINDOWS
+# define BSON_HAVE_RAND_R 0
+#else
+# define BSON_HAVE_RAND_R 1
+#endif
+#if BSON_HAVE_RAND_R != 1
+# undef BSON_HAVE_RAND_R
+#endif
+
+
+#define BSON_HAVE_STRLCPY 0
+#if BSON_HAVE_STRLCPY != 1
+# undef BSON_HAVE_STRLCPY
 #endif
 
 #endif /* BSON_CONFIG_H */
