@@ -18,10 +18,12 @@ import datetime
 import re
 import sys
 import uuid
+import io
 
 import bson
 from bson import json_util, EPOCH_AWARE
-from bson.binary import Binary, MD5_SUBTYPE, USER_DEFINED_SUBTYPE
+from bson.binary import (Binary, MD5_SUBTYPE, USER_DEFINED_SUBTYPE,
+                         UuidRepresentation)
 from bson.code import Code
 from bson.codec_options import CodecOptions
 from bson.decimal128 import Decimal128
@@ -45,17 +47,23 @@ from test import StringIO, unittest
 def to_object(bson_bytes):
     """Return deserialized object from BSON bytes"""
     return bson.BSON(bson_bytes).decode(CodecOptions(document_class=SON,
-                                                     tz_aware=True))
+                                                     tz_aware=True,
+                                                     uuid_representation=UuidRepresentation.PYTHON_LEGACY))
 
 
 def to_bson(obj):
     """Return serialized BSON string from object"""
-    return bson.BSON.encode(obj)
+    return bson.BSON.encode(obj, codec_options=CodecOptions(
+        uuid_representation=UuidRepresentation.PYTHON_LEGACY))
 
 
-def bsonjs_dumps(doc):
+def bsonjs_dumps(doc, mode=bsonjs.LEGACY):
     """Provide same API as json_util.dumps"""
-    return bsonjs.dumps(to_bson(doc))
+    return bsonjs.dumps(to_bson(doc), mode=mode)
+
+def bsonjs_dump(doc, file,  mode=bsonjs.LEGACY):
+    """Provide same API as json_util.dumps"""
+    return bsonjs.dump(to_bson(doc), file,  mode=mode)
 
 
 def bsonjs_loads(json_str):
@@ -63,6 +71,12 @@ def bsonjs_loads(json_str):
     return to_object(bsonjs.loads(json_str))
 
 
+BSONJS_JSON_OPTIONS = json_util.JSONOptions(
+    json_mode=json_util.JSONMode.CANONICAL,
+    uuid_representation=UuidRepresentation.PYTHON_LEGACY,
+    tz_aware=True)
+
+
 class TestBsonjs(unittest.TestCase):
 
     @staticmethod
@@ -75,9 +89,9 @@ class TestBsonjs(unittest.TestCase):
         # Check compatibility between bsonjs and json_util
         self.assertEqual(doc, json_util.loads(
             bsonjs.dumps(bson_bytes),
-            json_options=json_util.STRICT_JSON_OPTIONS))
+            json_options=BSONJS_JSON_OPTIONS))
         self.assertEqual(bson_bytes, bsonjs.loads(json_util.dumps(
-            doc, json_options=json_util.STRICT_JSON_OPTIONS)))
+            doc, json_options=BSONJS_JSON_OPTIONS)))
 
     def test_basic(self):
         self.round_trip({"hello": "world"})
@@ -143,9 +157,9 @@ class TestBsonjs(unittest.TestCase):
         res = self.round_tripped({"r": regex})["r"]
         self.assertEqual(unicode_options, res.flags)
 
-        # Some tools may not add $options if no flags are set.
-        res = bsonjs_loads('{"r": {"$regex": "a*b"}}')['r']
-        self.assertEqual(0, res.flags)
+        # Now that https://jira.mongodb.org/browse/CDRIVER-3773 is fixed
+        # this should no longer cause an error
+        bsonjs_loads('{"r": {"$regex": "a*b"}}')
 
         self.assertEqual(
             Regex(".*", "ilm"),
@@ -181,7 +195,9 @@ class TestBsonjs(unittest.TestCase):
                          uuid.UUID("f47ac10b-58cc-4372-a567-0e02b2c3d479")})
 
     def test_binary(self):
-        bin_type_dict = {"bin": Binary(b"\x00\x01\x02\x03\x04")}
+        bin0 = b"\x00\x01\x02\x03\x04"
+
+        bin_type_dict = {"bin": bin0}
         md5_type_dict = {
             "md5": Binary(b" n7\x18\xaf\t/\xd1\xd1/\x80\xca\xe7q\xcc\xac",
                           MD5_SUBTYPE)
@@ -232,10 +248,8 @@ class TestBsonjs(unittest.TestCase):
                           '{"a": {"$numberLong": "not-a-number"}}')
 
     def test_load_mongodb_extended_type_at_top_level(self):
-        self.assertRaises(ValueError, bsonjs.loads,
-                          '{"$numberLong": "42"}')
-        self.assertRaises(ValueError, bsonjs.loads,
-                          '{"$numberLong": "42", "a": 1}')
+        _ = bsonjs.loads('{"$numberLong": "42"}')
+        _ = bsonjs.loads('{"$numberLong": "42", "a": 1}')
         _ = bsonjs.loads('{"a": 1, "$numberLong": "42"}')
 
     def test_dumps_multiple_bson_documents(self):
@@ -278,5 +292,71 @@ class TestBsonjs(unittest.TestCase):
         not_file = {}
         self.assertRaises(AttributeError, bsonjs.load, not_file)
 
+    def test_mode(self):
+
+        json_str = '{ "test" : "me" }'
+        bson_bytes = bsonjs.loads(json_str)
+        self.assertRaises(ValueError, bsonjs.dumps, bson_bytes, mode=4)
+
+        # Test support for passing mode as positional argument
+        self.assertEqual(
+            '{ "regex" : { "$regex" : ".*", "$options" : "mx" } }',
+            bsonjs_dumps({"regex": Regex(".*", re.M | re.X)}, bsonjs.LEGACY))
+
+        self.assertEqual(
+            '{ "regex" : { "$regex" : ".*", "$options" : "mx" } }',
+            bsonjs_dumps({"regex": Regex(".*", re.M | re.X)},
+                         mode=bsonjs.LEGACY))
+        self.assertEqual(
+            '{ "regex" : { "$regularExpression" : { "pattern" : ".*", "options" : "mx" } } }',
+            bsonjs_dumps({"regex": Regex(".*", re.M | re.X)},
+                         mode=bsonjs.RELAXED))
+        self.assertEqual('{ "date" : { "$date" : "2020-12-16T00:00:00Z" } }',
+                        bsonjs_dumps({"date": datetime.datetime(2020, 12, 16)},
+                                     mode=bsonjs.RELAXED))
+        self.assertEqual('{ "date" : { "$date" : { "$numberLong" : "1608076800000" } } }',
+                         bsonjs_dumps({"date": datetime.datetime(2020, 12, 16)},
+                                      mode=bsonjs.CANONICAL))
+
+        # Test dump
+        with io.StringIO() as f:
+            bson_bytes = bsonjs.loads('{ "test" : "me" }')
+            self.assertRaises(ValueError, bsonjs.dump, bson_bytes, f,  mode=4)
+            
+        with io.StringIO() as f:
+            bsonjs_dump({"regex": Regex(".*", re.M | re.X)},
+                         f, bsonjs.LEGACY)
+            self.assertEqual(
+                '{ "regex" : { "$regex" : ".*", "$options" : "mx" } }',
+                f.getvalue())
+
+        with io.StringIO() as f:
+            bsonjs_dump({"regex": Regex(".*", re.M | re.X)},
+                        f, mode=bsonjs.LEGACY)
+            self.assertEqual(
+                '{ "regex" : { "$regex" : ".*", "$options" : "mx" } }',
+                f.getvalue())
+
+        with io.StringIO() as f:
+            bsonjs_dump({"regex": Regex(".*", re.M | re.X)},
+                        f, mode=bsonjs.RELAXED)
+            self.assertEqual(
+                '{ "regex" : { "$regularExpression" : { "pattern" : ".*", "options" : "mx" } } }',
+                f.getvalue())
+
+        with io.StringIO() as f:
+            bsonjs_dump({"date": datetime.datetime(2020, 12, 16)},
+                         f, mode=bsonjs.RELAXED)
+            self.assertEqual(
+                '{ "date" : { "$date" : "2020-12-16T00:00:00Z" } }',
+                f.getvalue())
+
+        with io.StringIO() as f:
+            bsonjs_dump({"date": datetime.datetime(2020, 12, 16)},
+                         f, mode=bsonjs.CANONICAL)
+            self.assertEqual(
+                '{ "date" : { "$date" : { "$numberLong" : "1608076800000" } '
+                '} }', f.getvalue())
+
 if __name__ == "__main__":
     unittest.main()
